using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using SetupGrid.Scripts;
using System.Reflection;
using SetupGrid.ScriptCommands;
using SetupGrid.Properties;

namespace SetupGrid
{
    public partial class SetupGridUC : UserControl, SetupGrid.ISetupGridUC
    {
        #region [ Constant(s) ]
        public const string SetupScriptsXml = "SetupScripts.xml"; 
        #endregion

        #region [ Private (s) ]
        private delegate void MethodDelegate();
        private delegate void VoidDelegate();
        private MethodDelegate methodDelegate;
        private IAsyncResult asyncResult;
        private bool _inSetupProgress = false;
        private bool _stopSetup = false;
        #endregion

        #region [ Constructor(s) ]
        public SetupGridUC()
        {
            InitializeComponent();
        } 
        #endregion

        #region [ Event(s) ]
        public event EventHandler ProgressStarted;
        public event EventHandler ProgressFinished; 
        #endregion

        #region [ Public Member(s) ]
        public bool HaltSetup
        {
            get
            {
                return _stopSetup;
            }
        }

        public void StartSetup()
        {
            SetDisable();
            if (setupDS.tblSetupScripts != null
                && setupDS.tblSetupScripts.Count > 0)
            {
                methodDelegate = new MethodDelegate(ExecuteScript);
                // will have the following BeginInvoke() and EndInvoke methods:
                _inSetupProgress = true;
                _stopSetup = false;
                asyncResult = methodDelegate.BeginInvoke(null, null);
            }
        }

        public void StopSetup(bool stop)
        {
            if (asyncResult != null && stop)
            {
                _inSetupProgress = false;
                methodDelegate.EndInvoke(asyncResult);
            }
            else
                lock (this)
                {
                    _stopSetup = true;
                }
        }

        public SetupDS DataSet
        {
            get
            {
                return setupDS;
            }
        }

        public bool InProgress
        {
            get
            {
                return _inSetupProgress;
            }
        }

        public void Load()
        {
            if (System.IO.File.Exists(SetupScriptsXml))
            {
                setupDS.Clear();
                setupDS.ReadXml(SetupScriptsXml);
            }
            else
                setupDS.WriteXml(SetupScriptsXml);
        }

        public void Save()
        {
            setupDS.WriteXml(SetupScriptsXml);
        } 
        #endregion

        #region [ Private Member(s) ]
        private void command_Progress(object sender, ProgressEventArgs e)
        {
            IBaseCommand command = (IBaseCommand)sender;
            SetupGrid.Properties.SetupDS.tblSetupScriptsRow scriptRow = command.Script;
            scriptRow.Progress = e.Percent;
            /// Call Refresh method of asyncSetupGrid using Invoke 
            SafeInvoke(asyncSetupGrid, Refresh);
        }

        private void SafeInvoke(Control ctrl, VoidDelegate delgt)
        {
            if (ctrl.InvokeRequired)
            {
                VoidDelegate d = new VoidDelegate(delgt);
                ctrl.Invoke(d, new object[] { });
            }
        }

        private void SetEnable()
        {
            if (ProgressFinished != null)
                ProgressFinished(this, new EventArgs());

            asyncSetupGrid.Cursor = Cursors.Default;
        }

        private void SetDisable()
        {
            asyncSetupGrid.Cursor = Cursors.AppStarting;
            foreach (SetupGrid.Properties.SetupDS.tblSetupScriptsRow row in this.setupDS.tblSetupScripts.Rows)
            {
                row.Progress = 0;
            }
            if (ProgressStarted != null)
                ProgressStarted(this, new EventArgs());
        }

        private void ExecuteScript()
        {
            System.Threading.Thread.CurrentThread.Priority = System.Threading.ThreadPriority.Lowest;
            IBaseCommand command = null;
            foreach (SetupGrid.Properties.SetupDS.tblSetupScriptsRow row in setupDS.tblSetupScripts.Rows)
            {
                if (!_stopSetup)
                {
                    try
                    {
                        object kind = Enum.Parse(typeof(ScriptKind), row.Kind);
                        switch ((ScriptKind)kind)
                        {
                            case ScriptKind.None:
                                break;
                            case ScriptKind.Method:
                                command = new MethodCommand(row, this);
                                break;
                            case ScriptKind.SQLScript:
                                command = new SQLScriptCommand(row, this);
                                break;
                            case ScriptKind.BatchFile:
                                command = new BatchFileCommand(row, this);
                                break;
                            default:
                                break;
                        }
                        if (command != null)
                        {
                            command.Progress += new ProgressEventHandler(command_Progress);
                            command.Execute();
                        }

                    }
                    catch (Exception ex)
                    {
                        row.Message = ex.Message;
                        row.Progress = -1;
                    }
                }
            }
            SafeInvoke(this, SetEnable);
            StopSetup(_inSetupProgress);
        }

        private void asyncSetupGrid_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            e.Cancel = true;
        }

        private void asyncSetupGrid_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            SetupDS.tblSetupScriptsRow row = ((DataRowView)asyncSetupGrid.Rows[e.RowIndex].DataBoundItem).Row as SetupDS.tblSetupScriptsRow;
            if (row.Progress < 0) // An exception has happened
            {
                if (asyncSetupGrid.Columns[e.ColumnIndex].HeaderText.ToLower().Equals("progress"))
                    MessageBox.Show(row.Message, "Error");
            }
        }

        private void asyncSetupGrid_CellMouseEnter(object sender, DataGridViewCellEventArgs e)
        {

        }

        private void asyncSetupGrid_CellMouseMove(object sender, DataGridViewCellMouseEventArgs e)
        {
            try
            {
                SetupDS.tblSetupScriptsRow row = ((DataRowView)asyncSetupGrid.Rows[e.RowIndex].DataBoundItem).Row as SetupDS.tblSetupScriptsRow;
                if (row.Progress < 0) // An exception has happened
                {
                    if (asyncSetupGrid.Columns[e.ColumnIndex].HeaderText.ToLower().Equals("progress"))
                        asyncSetupGrid.Cursor = Cursors.Hand;
                    else
                        asyncSetupGrid.Cursor = Cursors.Default;
                }
                else
                    asyncSetupGrid.Cursor = Cursors.Default;
            }
            catch (Exception ex)
            {
                //throw ex;
            }
        } 
        #endregion

    }
}
